/*!
 * gameTempla.js
 * author: Brent M. Dingle
 * last modified: 2015
 * Template html5 canvas game --- still in development, Alpha version 0.2
 *
 * All canvas elements are to be resized with the screen
 * -- for details see the 'window handling' stuff at the bottom of this file
 *
 * This can also support: 
 *    An intro menu screen
 *    Background and sound effects
 *    and possibly an intro cut scene (.mp4) before the game starts
 * These along with a few other features =)
 *
 * Aside: If game over pops immediately you likely have a "this." missing
 */

// ----------------------------------------------------------------------------
//                                                          GLOBAL VARs
// ----------------------------------------------------------------------------
// The below must match the id's used in the HTML file
// ----------------------------------------------------------------------------
var CANVAS_MENU_ID    = "canvasMenu";
var CANVAS_CREDITS_ID = "canvasCredits";
var CANVAS_GAME_ID    = "canvasGame";
var INTROCUT_ID       = "introCut";
var LOADING_ID        = "loadingMessage";

var CANVAS_WIDTH      = 640;
var CANVAS_HEIGHT     = 480;

var OK_MIN_SCREEN_RATIO = 1.33;  // round down CANVAS_WIDTH / CANVAS_HEIGHT
var OK_MAX_SCREEN_RATIO = 1.35;  // round up + 0.01

var DISABLE_RESIZING = false;     // set to true if you do NOT want auto-canvas resizing


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theGame Object											theGame Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theGame = 
{
	// ----------------------------------------------------------------------------
	// Pseudo-constants
	// ----------------------------------------------------------------------------
	STATE_MENU:         0,
	STATE_CREDITS:      1,
	STATE_PLAYING:      5,
    STATE_INTROCUT:   101,
    STATE_GAMEPAUSED: 888,
    STATE_GAMEOVER:   999,
    
    // Game specific constants follow
    
    
	// ----------------------------------------------------------------------------
	// Variables
	// ----------------------------------------------------------------------------
	gameState: this.STATE_PLAYING,   // check Init for actual value
	isInitialized:  0,   // 0=false, 1=true to indicate if Init() function was called
    
    startTime: 0,
    curTime:   0,
    prevTime:  0,
    delay:     0, //0.03,  // 0 means run as fast as possible

    width:  CANVAS_WIDTH,
    height: CANVAS_HEIGHT,
    
    menu:    new EZmenu('gameTemplaMenu.png'),         // TODO: see files --> gameTemplaMenu.js
    credits: new EZcredits('gameTemplaCredits.png'),   //       and gameTemplaCredits.js
                                                       //       to 'personalize' the game
    
    // ------------------ more game specific items
    score:          0,
    highScore:      0,
    alive:          1,      // 0 for false, 1 for true

	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
	Init: function ()
	{
        // Init Local Canvas variables
		var gameCanvas = document.getElementById(CANVAS_GAME_ID);
		var gameCTX = gameCanvas.getContext("2d");
        
        this.height = gameCanvas.height;
        this.width  = gameCanvas.width;

        // html must define a button with id: playAgainButton and another with id mainMenuButton
        document.getElementById('playAgainButton').addEventListener('click', 
                                                        function(evt) 
                                                        {
                                                            theGame.ResetPlay();
                                                            evt.cancelBubble=true; // prevent click going through to new game
                                                        } );
        document.getElementById('resumePlayButton').addEventListener('click', 
                                                        function(evt) 
                                                        {
                                                            document.getElementById('gameOverOverlay').style.display = 'none';
                                                            document.getElementById('gamePauseDiv').style.display = 'none';
                                                            theGame.gameState = theGame.STATE_PLAYING;
                                                            evt.cancelBubble=true; // prevent click going through to new game
                                                        } );
        document.getElementById('mainMenuButton').addEventListener('click', theGame.SwitchToMenu);
        document.getElementById('mainMenuButton2').addEventListener('click', theGame.SwitchToMenu);

        // Menu Item stuff - done a little different
        var menuCanvas = document.getElementById(CANVAS_MENU_ID);
        menuCanvas.addEventListener('click', this.menu.doMenuClick, false);
        menuCanvas.addEventListener('mousemove', this.menu.doMouseMove, false);

        var creditCanvas = document.getElementById(CANVAS_CREDITS_ID);
        creditCanvas.addEventListener('click', this.credits.doMenuClick, false);
        creditCanvas.addEventListener('mousemove', this.credits.doMouseMove, false);

        // Reset all other variables for beginning of play
        // Note difference between initialize and "play again"
        theGame.ResetPlay();
        
        // Can now make the initial loading screen disappear
        // (menu may put up another version, as well as cut scene videos)
		document.getElementById(LOADING_ID).style.display = 'none';
        
        // Set state to menu and Make menu visible
        this.gameState = this.STATE_MENU;
        menuCanvas.style.display = 'block';
        
        this.isInitialized = 1;
    },

	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    GameOver: function()
    {
        // Display the game over div
        document.getElementById('gameOverDiv').style.display = 'block';
        document.getElementById('gameOverOverlay').style.display = 'block';
        
        // disable the canvas 
        // -- if you want the final scene to remain 
        //    visible behind the "game over" message then comment out the below line
        //document.getElementById(CANVAS_GAME_ID).style.display = 'none';

        // Set game state
        this.gameState = this.STATE_GAMEOVER;
        
        // Update high score
        if (this.score > this.highScore)
        {
            this.highScore = this.score;
        }
        
        // button event handlers move us out of this
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    PauseGame: function()
    {
        // Display the correct div
        document.getElementById('gamePauseDiv').style.display = 'block';
        document.getElementById('gameOverOverlay').style.display = 'block';

        // Set game state
        this.gameState = this.STATE_GAMEPAUSED;
        
    },

	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    SwitchToMenu: function()
    {
        theGame.ResetPlay();
        var menuCanvas = document.getElementById(CANVAS_MENU_ID);
        var gameCanvas = document.getElementById(CANVAS_GAME_ID);
        var creditsCanvas = document.getElementById(CANVAS_CREDITS_ID);
        gameCanvas.style.display    = "none";
        menuCanvas.style.display    = "block";
        creditsCanvas.style.display = "none";
        theGame.gameState = theGame.STATE_MENU;
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ResetPlay: function()
    {
        // Hide the game over div and the paused div
        document.getElementById('gameOverDiv').style.display = 'none';
        document.getElementById('gameOverOverlay').style.display = 'none';
        document.getElementById('gamePauseDiv').style.display = 'none';

        // enable the canvas (if pause/game over disabled it)
        //document.getElementById(CANVAS_GAME_ID).style.display = 'block';
        
        // set state to playing
        this.gameState = this.STATE_PLAYING;
        
        // reinit the game-play variables
        this.score = 0;
        this.alive = 1;
        
    },
    
	// ----------------------------------------------------------------------------
    // deltaTime expected to be in seconds
	// ----------------------------------------------------------------------------
    UpdatePlay: function(deltaTime)
    {
		var gameCanvas = document.getElementById(CANVAS_GAME_ID);
		var gameCTX = gameCanvas.getContext("2d");

        // If we allow window resize and auto adjust - must also
        // update this.height and width (either here or onResize)
        this.height = gameCanvas.height;
        this.width  = gameCanvas.width;
        
        // Clear the screen
        gameCTX.fillStyle = "#000";
        gameCTX.fillRect(0, 0, this.width, this.height);
        
        // Write the scores
        gameCTX.fillStyle = "#FFF";
        gameCTX.fillText("HS: " + this.highScore, 0.85*this.width, 10);
        gameCTX.fillText("Score: " + this.score, 0.40*this.width, 10);
        
        if (this.alive != 0)  // if player is still alive
        {

        } 
		else // player has died
        {
            // Primitive style game over commented 
			// gameCTX.fillText("GAME OVER", this.width/2-this.shapeWidth, this.height/2);
            
            // Display fancy Game Over
            theGame.GameOver();
		}
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ProcessKeydown: function(evt)
    {
        switch (evt.keyCode) 
        {
            // ------------------------------------------------- arrow left
            case 37:
            break;

            // ------------------------------------------------- arrow right
            case 39:
            break;

            // ------------------------------------------------- arrow up
            case 38:				
            break;		
            
            // ------------------------------------------------- spacebar
            case 32:				
            break;

        }
    },

	// ------------------------------------------------------------------------
	// ------------------------------------------------------------------------
	getMousePos: function(canvas, evt)
	{
		var rect = canvas.getBoundingClientRect();
		return 	{
					x: Math.round( (evt.clientX-rect.left)/(rect.right-rect.left)*canvas.width ),
					y: Math.round( (evt.clientY-rect.top)/(rect.bottom-rect.top)*canvas.height )
				};
	},
	
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ProcessOnClick: function(evt)
    {
        var gameCanvas = document.getElementById(CANVAS_GAME_ID);
        var mousePos = theGame.getMousePos(gameCanvas, evt);
        this.ProcessTouchClick(mousePos.x);
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ProcessTouchClick: function(eventPosX)
    {
    },
    
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
    ProcessOnTouch: function(evt)
    {
        var gameCanvas = document.getElementById(CANVAS_GAME_ID);
        var touchPos = theGame.getTouchPos(gameCanvas, evt);
        this.ProcessTouchClick(touchPos.x);
    },
    
	// ------------------------------------------------------------------------
	// ------------------------------------------------------------------------
	getTouchPos: function(canvas, evt)
	{
		//var touch = evt.targetTouches[0];
		var touch = evt.changedTouches[0];
		var rect = canvas.getBoundingClientRect();
		return 	{
					x: Math.round( (touch.pageX-rect.left)/(rect.right-rect.left)*canvas.width ),
					y: Math.round( (touch.pageY-rect.top)/(rect.bottom-rect.top)*canvas.height )
				};
	},
    
    
	// ----------------------------------------------------------------------------
	// Update Based on this.gameState
    // Notice for this example we have some different approaches in design/implementation
    // One for Game Over and another for the Menu (and maybe one for credits)
    // each has its own way of changing game state, which eventually comes back to here
    //
    // Also notice this.delay can be used to "force" a specific 'fastest' rate possible
    // Set this.delay to zero to run 'as fast as possible'
	// ----------------------------------------------------------------------------
	Update: function ()
	{
		if (this.gameState == this.STATE_PLAYING)
		{
            newDate  = new Date();
            this.curTime  = newDate.getTime();  // end time in milliseconds
            deltaTime = (this.curTime - this.prevTime) / 1000.0;    // delta time in seconds
            
            if (deltaTime >= this.delay)
            {
                this.prevTime = this.curTime;
                theGame.UpdatePlay(deltaTime);
            }
		}
		else if (this.gameState == this.STATE_MENU)
		{
            theGame.menu.DrawMenu();
		}
		else if (this.gameState == this.STATE_CREDITS)
		{
            theGame.credits.DrawCredits();
		}
        // GameOver and Paused states will handle themselves through button responses
        else
        {
            //console.log("ERROR: Unrecognized game state in Update")
        }
    },
    
	// ----------------------------------------------------------------------------
	// Draw Based on this.gameState
	// Note: This game does the drawing in the update function,
    //       it works... for now
	// ----------------------------------------------------------------------------
	// Draw: function ()
	// {
		// if (this.gameState == this.STATE_PLAYING)
		// {
			// //theGame.Draw();    <--------- TODO separate draw and update if possible
		// }
		// else if (this.gameState == this.STATE_MENU)
		// {
			// // theGame.DrawMenu();
		// }
        // else if (this.gameState == this.STATE_GAMEOVER)
        // {
            // // do nothing - game over already displayed by GameOver()
        // }
		// else
		// {
			// console.log("ERROR: Draw() -- unknown gameState = " + this.gameState );
		// }
	// },
	
};   // end of theGame variable/object

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Window Handling Stuff
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
window.addEventListener("resize", doResize, false);
window.addEventListener("keydown", doKeydown, false);
window.addEventListener("click", doClick, false);
//window.addEventListener("touchstart", doTouchStart, false);
window.addEventListener("touchend", doTouchEnd, false);

// ----------------------------------------------------------------------------
//                                                             doResize
//
// Need to know the canvas percentage of window for additional measure scaling
// if only have 80% of the height then need to scale to window.height*0.8
// i.e. other elements in the window are not scaled here
// OR must scale everything on the window
// ----------------------------------------------------------------------------
function doResize() 
{
    if (!DISABLE_RESIZING)
    {
        var canvas1 = document.getElementById(CANVAS_MENU_ID);
        var canvas2 = document.getElementById(CANVAS_CREDITS_ID);
        var canvas3 = document.getElementById(CANVAS_GAME_ID);
        
        // Size things
        UpdateCanvas(canvas1);
        UpdateCanvas(canvas2);
        UpdateCanvas(canvas3);

        // If game has been initialized then force a game redraw
        // Note the game itself is NOT updated here... just changed the size of view
        if (theGame.isInitialized != 0)
        {
            //theGame.Draw();   not needed in this case
            theGame.Update();
        }
    }
};

// ----------------------------------------------------------------------------
//                                                               UpdateCanvas
// ----------------------------------------------------------------------------
function UpdateCanvas(canvas) 
{
    // Oddly it appears sometimes - if the canvas display = none, 
    // the browser may ignore the below changes - which means user must resize again to get pretty
    var tempDisplay = canvas.style.display;
    canvas.style.display = "block";
    
	//console.log("UpdateCanvas Called");
	
	// Here's the tricky part 
	// - Set the width and height of the canvas's coordinate system
	// - if not specified it defaults to width=300 and height=150
	// This is the size you "pretend" your world is (i.e. optimal)
	// 
	canvas.width = CANVAS_WIDTH;   // World Coordinate system size should NOT change
	canvas.height = CANVAS_HEIGHT; // just the viewing size of it (i.e. style)
	
	// DEBUG: console.log(canvas.width + " " + canvas.height);
	// DEBUG: console.log("W " + window.innerWidth + " " + window.innerHeight);
    // DEBUG: console.log("DEBUG: text height = " + introText.clientHeight); div size doesn't include spacing between divs
    
	// Now we consider the actual display (the style width and height)
	// -- this is the size of the window you are using to view your world

    // canvas.style.width = window.innerWidth + 'px';       // This line and the next resize things
    // canvas.style.height = window.innerHeight + 'px';     // but do not maintain scale ratio
	
	// The below scale the game and maintain a ratio of 800x480 
	// by "adding" empty space to the right and bottom of the canvas as appropriate
	// You may need to put a box around the canvas to better see this
	
    var rect = canvas.getBoundingClientRect();

	var gameWidth = window.innerWidth;
	var gameHeight = window.innerHeight - rect.top; // Recall: introText.clientHeight doesn't include spacing between divs
	var scaleToFitX = gameWidth / canvas.width;   // canvas.width and height just set above
	var scaleToFitY = gameHeight / canvas.height;

	var currentScreenRatio = gameWidth / gameHeight;
	var optimalRatio = Math.min(scaleToFitX, scaleToFitY);

	canvas.style.position = "fixed";
	
    // For ref: width = 800, height = 450 then 800/450 ~= 1.78, bound by 1.77 and 1.79
	if (currentScreenRatio >= OK_MIN_SCREEN_RATIO && currentScreenRatio <= OK_MAX_SCREEN_RATIO) 
    {
		canvas.style.width = gameWidth + "px";
		canvas.style.height = gameHeight + "px";
		canvas.style.left = 0 + "px";
	}
	else 
    {
		canvas.style.width = canvas.width * optimalRatio + "px";
		canvas.style.height = canvas.height * optimalRatio + "px";
		canvas.style.left = ( (gameWidth - (canvas.width * optimalRatio)) / 2 ) + "px";

	}
    
    canvas.style.display = tempDisplay;
};

// ----------------------------------------------------------------------------
//                                                                doKeydown
// ----------------------------------------------------------------------------
function doKeydown(e)
{ 
	if (e.keyCode == 27)
	{
        if (   (theGame.gameState != theGame.STATE_MENU)
            && (theGame.gameState != theGame.STATE_CREDITS) )
        {
            // var menuCanvas = document.getElementById(CANVAS_MENU_ID);
            // var gameCanvas = document.getElementById(CANVAS_GAME_ID);
            // gameCanvas.style.display="none";
            // menuCanvas.style.display="block";
            theGame.PauseGame();
        }
	}
    else
    {
        theGame.ProcessKeydown(e);
    }
	// Depending on the game may need a keylistener for keyup and keypress also
};
// ----------------------------------------------------------------------------
function doClick(e)
{
    theGame.ProcessOnClick(e);
    
     // disable browser's default behaviour and prevent multiple event triggers
    e.preventDefault();
    
    return false;		
};
// ----------------------------------------------------------------------------
function doTouchEnd(e)
{
    if (theGame.gameState == theGame.STATE_PLAYING)
    {
        theGame.ProcessOnTouch(e);

        // disable browser's default behaviour and prevent multiple event triggers
        e.preventDefault();
    }
    
    return false;		
};


// ----------------------------------------------------------------------------
//                                                        window.ONLOAD
// ----------------------------------------------------------------------------
window.onload = function()
{
    if (!DISABLE_RESIZING)
    {
        // Size all canvas to correct Ratio before doing anything 
        // Needed for resizing and centring to work correctly from the start
        doResize();
    }
	
	// Initialize the game
	theGame.Init();

	// Run the game (start the game loop)
	runGame();
};

// ----------------------------------------------------------------------------
//                                                         the game 'loop'
// Event handlers/listeners cover the game input
// ----------------------------------------------------------------------------
function runGame()
{
	//theGame.Draw();    not needed for this game          
	theGame.Update();

    // Be certain to call runGame again	
	requestAnimationFrame(runGame);
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end of file
